/******************************************************************
 *
 * Name:	Plugin_procs.c
 * 
 * Description:
 * 
 * Author:	Tomasz Stachowiak
 *
 * Date:	18/07/96
 *
 * Change Log:
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/types.h>

#include <dve/dve.h>

#include <Xm/Xm.h>

#include <sys/time.h>
#include <dmedia/audio.h>


#include "TSAudioPluginMotif.h"
#include "AudioPluginProtocol.h"
#include "gsm.h"
#include "adpcm.h"


extern ALport audio_output_ports[];
extern ALconfig conf[];

extern DveSession dveSession;       /* global variables */
int current_state;
XtAppContext app;
XtWorkProcId workproc;
extern code_type;

gsm handle;

gsm_init()
  {
     handle = gsm_create();
  }

gsm_close()
  {
     gsm_destroy(handle);
  }

init_audio_output_ports()
  {
     int i;

     for(i=0; i<100; i++)
	audio_output_ports[i] = 0;
  }

close_audio_output_ports()
  {
     int i;

     for(i=0; i<100; i++)
       if(audio_output_ports[i] != 0)
	 {
	   ALcloseport(audio_output_ports[i]);
	   ALfreeconfig(conf[i]);
	 }
  }




/**********************************************************************
 *
 * Functions kept for OpenDVE_procs.c compatibility
 *
 *********************************************************************/
void QueryPluginStatus(DveConfParticipant *part)
{
}

void ProcessClipboardData(char *format, char *data, int length)
{
}



/********************************************************************
 *
 * Frames management
 *
 *******************************************************************/
void DecodeGSMFrame(gsm_frame *frames, int part)
{
   char buffer[5][320];
   struct timeval rec_time;
   int i;
  
   for(i=0; i<5; i++)
     {
	gsm_decode(handle, frames[i], buffer[i]);
     }
   
   ALwritesamps(audio_output_ports[part], buffer, 800);
 }




void
SendGSMFrame()
{
   audioPluginMessageStructure *msg;
   char sig[5][320];
   int size;
   struct timeval send_time;
   int i;

   msg = (audioPluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(audioPluginMessageStructure));
   if (!msg)
   {
     printf("Message allocation error\n");
     return;
   }
 
   /* Get the audio from mic. */
   GetFrameFromMic(sig);
   for(i=0; i<5; i++)
     {
	gsm_encode(handle, sig[i], ((gsm_frame *)(msg->buf))[i]);
     }

   if(SendPluginMessage(dveSession,
                           AUDIO_PLUGIN_PORT, GSM, msg,
                           sizeof(audioPluginMessageStructure)) < 0)
		printf("Sending failure\n");

}

void DecodeADPCMFrame(char *inbuf, int part)
  {
   char buffer[1600];
   struct adpcm_state mystate;

   mystate.valprev = 0;
   mystate.index = 0;
   adpcm_decoder(inbuf, buffer, 800, &mystate);
   
   ALwritesamps(audio_output_ports[part], buffer, 800);
 }


void
SendADPCMFrame()
{
   audioPluginMessageStructure *msg;
   char sig[1600];
   int size;
   struct adpcm_state mystate;

   msg = (audioPluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(audioPluginMessageStructure));
   if (!msg)
   {
     printf("Message allocation error\n");
     return;
   }
 
   /* Get the audio from mic. */
   GetFrameFromMic(sig);
   mystate.valprev = 0;
   mystate.index = 0;
   adpcm_coder(sig, msg->buf, 800, &mystate);

   if(SendPluginMessage(dveSession,
                           AUDIO_PLUGIN_PORT, ADPCM, msg,
                           sizeof(audioPluginMessageStructure)) < 0)
		printf("Sending failure\n");

}

void SendFrame()
{
  switch(code_type)
    {
    case GSM:
      SendGSMFrame();
      break;

    case ADPCM:
      SendADPCMFrame();
      break;
    }

  if(current_state == START_STATE)
    workproc = XtAppAddWorkProc(app, SendFrame, NULL);
}


/************************************************************************
 *
 * Conference mangement functions
 *
 ***********************************************************************/
void 
MemberLeft(DveConfParticipant member)
{
   fprintf(stderr, "User Left the Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

void 
MemberAdded(DveConfParticipant member)
{
   fprintf(stderr, "User Added to Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}


/**************************************************************************
 *
 * Function processing incoming messages
 *
 *************************************************************************/
void 
ProcessPluginMessage(u_int32 messageType, void *data, int data_size, 
		     DveUserHandle sender) 
{
   audioPluginMessageStructure message;

   memcpy(&message, data, sizeof(audioPluginMessageStructure));

   switch (messageType){
      case ADPCM:
         if(audio_output_ports[sender] == 0)
	   {
	     OpenOutPort(sender);
	   }
         DecodeADPCMFrame(message.buf, sender);
	 break;

      case GSM:
         if(audio_output_ports[sender] == 0)
	   {
	     OpenOutPort(sender);
	   }

	 DecodeGSMFrame(message.buf, sender);
	 break;
   }
}


/*********************************************************************
 *
 * Plugin management functions
 *
 ********************************************************************/
void 
ShowPlugin(void)
{
   fprintf(stderr, "ShowPlugin\n");
}

void 
HidePlugin(void)
{
   fprintf(stderr, "HidePlugin\n");
}

void 
CleanupPlugin(void)
{
  CloseSpeaker();
}


/************************** END Plugin_procs.c **************************/






